/****************************************************************************
*  HILITE.C       Clipper 5.x callable function
*
*  Usage:
*
*     HILITE( <nT>, <nL>, <nB>, <nR>, <"STRING">, <"COLOR"> ) --> nOccurances
*
*  Hilite changes the color (attribute) of all matching words/phrases
*  within the specified coordinates and returns the number of occurances.
*  Hilite is not case sensitive.
*
*  Hilite can be usefull when you want to emphasize information on the
*  screen when you don't necessarily know it's location, such as in
*  terminal emulation programs.  I also use it to hilite the words used as
*  search criteria when browsing or otherwise displaying the resulting
*  records.
*
*  Compatibility is 100% when compiled with MS C 5.x
*  with the following command line switches:
*
*  CL HILITE.C /c /AL /Oalt /Zl /FPa /Gs
*
*  May be compiled using Quick C 2.x if no 3rd party
*  LIB's are used and if no floating point number functions
*  are added:
*
*  CL /c /AL /Oalt /Zl /Gs HILITE.C
*
*  Jim Green
*  1421 Waterford Place
*  Garland, TX 75044
*  214-495-1420
*  10/19/91
*
*****************************************************************************/
#include "c:\clipper5\include\extend.h"
#include <ctype.h>
#include <bios.h>

/* Prototypes for local functions */
int color_xl( char *);
void colorit( int, int, int, int);

char *far vid_mem;

/***************************
*  HILITE()
***************************/
CLIPPER hilite()
{
   register int x, y;
   int t, l, b, r, matching, marker, attr, vmode, hits;
   char *strg, *color, far *v, *teststr, c;
   union REGS reg;

   t = _parni(1);
   l = _parni(2);
   b = _parni(3);
   r = _parni(4);
   strg = _parc(5);
   color= _parc(6);

   hits=0;

   /* Make all upper case */
   for( teststr = strg; *teststr; teststr++ ) *teststr = toupper( *teststr);

   /* Convert Clipper color string to attribute */
   attr = color_xl(color);

   /* Get video mode */
   reg.h.ah = 15;
   vmode = int86(0x10, &reg, &reg) & 255;

   if (vmode==7)     /* Monochrome? */
      vid_mem = (char far*) 0xB0000000;
   else
      vid_mem = (char far *) 0xB8000000;

   for ( y=t; y<=b; y++)            /* Loop from top to bottom rows */
   {
      v = vid_mem + (y*160) + l*2;  /* compute the address */
      teststr = strg;
      matching = FALSE;
      for ( x=l; x<=r; x++)      /* Loop for each row, left-to-right */
      {
         c = toupper(*v);                    /* Make display char upper */
         if ( *teststr == c )                /* Does character match? */
         {
            ++teststr;           /* Increment to next char in test string */
            if (!matching)       /* Is this the first matching character? */
            {
               matching = TRUE;
               marker = x;       /* Mark the beginning of the matching string */
            }
            if (*(teststr) == NULL) /* NULL? End of string? */
            {
               colorit(y,marker,strlen(strg),attr);   /* Hilite the match */
               matching = FALSE; /* Reset for next match */
               teststr = strg;   /* reset pointer */
               ++hits;           /* Count the 'hits' */
            }
         }
         else  /* Character does't match */
         {
            matching = FALSE; /* reset matching flag */
            teststr = strg;   /* reset pointer */
         }  /* end if/else */
         *v++;          /* skip past attr */
         *v++;          /* position on next character */
      }  /* End for */
   }  /* End for */
   _retni(hits);
}

/*************************************************************
 * COLORIT()      Local function to write color attribute to
 *    Jim Green   the specified area in the display memory
 *************************************************************/
void colorit(int x, int y, int cnt, int attrib)
{
   register int i;
   char far *v;

   v = vid_mem;
   v += (x*160) + y*2;     /* compute the address */
   for(i=1; i<=cnt; i++)
   {
      *v++;                /* increment past character */
      *v++ = attrib;       /* write the attribute */
   }
}

/********************************************************************
* COLOR_XL  Local function to convert a Clipper color (a string) into
*           an integer color attribute
*  Jim Green
********************************************************************/
int color_xl( char *color )
{
   register i=0;
   int attr = 0x00;
   char ch;
   while (color[i] && color[i] != '/')
   {
      ch = color[i++];
      switch(ch)
      {
         case '+':
            attr |= 0x08;
            break;
         case '*':
            attr |= 0x80;
            break;
         case 'R':
         case 'r':
            attr |= 0x04;
            break;
         case 'G':
         case 'g':
            attr |= 0x02;
            break;
         case 'B':
         case 'b':
            attr |= 0x01;
            break;
         case 'W':
         case 'w':
            attr |= 0x07;
            break;
      }
   }

   while (color[i])
   {
      ch = color[i++];
      switch(ch)
      {
         case 'R':
         case 'r':
            attr |= 0x40;
            break;
         case 'G':
         case 'g':
            attr |= 0x20;
            break;
         case 'B':
         case 'b':
            attr |= 0x10;
            break;
         case 'W':
         case 'w':
            attr |= 0x70;
            break;
      }
   }
   return attr;
}

